#!/usr/bin/env python
# encoding: utf-8
"""
@time: 2021/5/26 10:19
@desc: 电能管理界面
"""

from abstract_api import AbstractApi
from event_storage import EventStorage
import datetime
from datetime import timedelta
import pandas
from log import OutPutLog
import calendar

class ElectricMangement(AbstractApi):
    """电能管理 界面的接口"""
    def __init__(self):
        self._log = OutPutLog()
        self._operate_mysql = EventStorage()
        self._now = datetime.datetime.now()                          # 2021-05-26 10:33:13.252703
        self._today_date = self._now.strftime("%Y-%m-%d")   # 2021-05-26
        self._yesterday_date = (self._now + timedelta(days=-1)).strftime("%Y-%m-%d")  # 2021-05-25
        self._month_data = self._now.strftime("%Y-%m")      # 2021-05
        self._last_month_data = datetime.date(self._now.year, self._now.month, 1) - datetime.timedelta(1)
        self._year_data = self._now.strftime("%Y")

    def operation(self, request):

        # ===============================
        # ========== 用电的计算 ==========
        # ===============================

        # 当日、当月用电趋势（kw-h）、 当日实时用电（kw-h）、当月累计用电（kw-h）
        # unit_electric = "正向有功电能值"
        # sql_each_device = "SELECT serial_number, device_name FROM data_point_tbl WHERE io_point_name=\'%s\';" % (unit_electric)
        unit_electric = "kWh"
        sql_each_device = "SELECT serial_number, device_name FROM data_point_tbl WHERE unit=\'%s\' AND device_name like '%%_electric';" % (unit_electric)
        res_each_device = self._operate_mysql.execute_sql(sql_each_device)
        ele_day_all = {}            # 每天的用电总量
        ele_minute_all = {}         # 每小时的用电总量
        ele_year_all = 0           # 当年的用电总量

        # 查询每台设备
        for each1 in res_each_device:
            ele_serial_number = "c" + str(each1["serial_number"])
            ele_table_name = "table_" + each1["device_name"]
            # 返回各设备每天的用电量(本月)
            sql_ele_day = "SELECT DATE_FORMAT(times, '%%Y-%%m-%%d') AS t, MAX(%s) AS num FROM %s WHERE times>\'%s\' GROUP BY t;" % (ele_serial_number, ele_table_name, self._month_data)
            # 返回各设备上个月最后一天的用电总量
            # sql_ele_last_month = "SELECT DATE_FORMAT(times, '%%Y-%%m') AS t, MAX(%s) AS num FROM %s WHERE times<\'%s\' GROUP BY t ORDER BY t DESC LIMIT 1;" % (ele_serial_number, ele_table_name, self._month_data)
            sql_ele_last_month = "SELECT times AS t, %s AS num FROM %s WHERE times<\'%s\' ORDER BY t DESC LIMIT 1;" % (ele_serial_number, ele_table_name, self._month_data)
            res_ele_day = self.calculation_each_use(sql_ele_day,  sql_ele_last_month)
            for each_ele in res_ele_day:
                if res_ele_day[each_ele] > 0:
                    if each_ele in ele_day_all.keys():
                        ele_day_all[each_ele] = ele_day_all[each_ele] + res_ele_day[each_ele]
                    else:
                        ele_day_all[each_ele] = res_ele_day[each_ele]


            # 返回各设备去年最后的用电量
            sql_ele_last = "SELECT %s AS num FROM %s WHERE %s>0 ORDER BY times DESC LIMIT 1;" % (ele_serial_number, ele_table_name, ele_serial_number)
            ele_last = self._operate_mysql.execute_sql(sql_ele_last)
            sql_ele_last_year = "SELECT %s AS num FROM %s WHERE times<\'%s\' AND %s>0 ORDER BY times DESC LIMIT 1;" % (ele_serial_number, ele_table_name, self._year_data, ele_serial_number)
            ele_last_year = self._operate_mysql.execute_sql(sql_ele_last_year)
            last_data = 0
            last_year_data = 0
            if isinstance(ele_last, list) and len(ele_last) == 1:
                last_data = ele_last[0]["num"]
            if isinstance(ele_last_year, list) and len(ele_last_year) == 1:
                last_year_data = ele_last_year[0]["num"]
            ele_year_all += (last_data - last_year_data) 


            # 返回各设备每小时的用电量(昨天和今天)
            # sql_ele_minute = "SELECT DATE_FORMAT(times, '%%Y-%%m-%%d %%H') AS t, MAX(%s) AS num FROM %s WHERE times>=\'%s\' GROUP BY t;" % (ele_serial_number, ele_table_name, self._yesterday_date)
            sql_ele_minute = "SELECT DATE_FORMAT(times, '%%Y-%%m-%%d %%H') AS t, MAX(%s) AS num FROM %s WHERE times>=\'%s\' AND %s>0 GROUP BY t;" % (ele_serial_number, ele_table_name, self._yesterday_date, ele_serial_number)
            # 返回各设备前天最后的用电量
            # sql_ele_before_yestyeday = "SELECT times, %s AS num FROM %s WHERE times<\'%s\' ORDER BY times DESC LIMIT 1;" % (ele_serial_number, ele_table_name, self._yesterday_date)
            sql_ele_before_yestyeday = "SELECT times, %s AS num FROM %s WHERE times<\'%s\' AND %s>0 ORDER BY times DESC LIMIT 1;" % (ele_serial_number, ele_table_name, self._yesterday_date, ele_serial_number)
            res_ele_minute = self.calculation_each_use(sql_ele_minute, sql_ele_before_yestyeday)
            for each_ele in res_ele_minute:
                if each_ele in ele_minute_all.keys():
                    ele_minute_all[each_ele] = ele_minute_all[each_ele] + res_ele_minute[each_ele]
                else:
                    ele_minute_all[each_ele] = res_ele_minute[each_ele]


        # 当月用电趋势
        ele_time_day = []      # 当月每天的日期
        ele_day_use = []       # 当月每天的用电量
        for k in sorted(ele_day_all):
            ele_time_day.append(k)
            ele_day_use.append('%.1f' % ele_day_all[k])

        # # 昨天和今天的趋势分组
        ele_minute_trend = self.group_today_yesterday(ele_minute_all)
        ele_time_minute = ele_minute_trend[0]       # 昨日和今日的时刻（按小时划分）
        ele_today_ues = ele_minute_trend[1]             # 今天每小时的用电量
        ele_yesterday_ues = ele_minute_trend[2]         # 昨天每小时的用电量


        # ===============================
        # ========== 流量的计算 ==========
        # ===============================
        flow_serial_number = "c361"
        flow_table_name = "table_llj"

        # # ① 今年流量趋势（t）
        flow_time_day = []      # 今年的日期
        flow_day_use = []       # 今年日期对应的流量
        # 返回今年的累计流量
        sql_flow_day = "SELECT DATE_FORMAT(times, '%%Y-%%m-%%d') AS t, MAX(%s) AS num FROM %s WHERE times>\'%s\' GROUP BY t;" % (flow_serial_number, flow_table_name, self._year_data)
        # 返回去年最后的流量记录
        # sql_flow_last_month = "SELECT DATE_FORMAT(times, '%%Y-%%m') AS t, MAX(%s) AS num FROM %s WHERE times<\'%s\' GROUP BY t ORDER BY t DESC LIMIT 1;;" % (flow_serial_number, flow_table_name, self._month_data)
        sql_flow_last_month = "SELECT times, %s AS num FROM %s WHERE times<\'%s\' ORDER BY times DESC LIMIT 1;" % (flow_serial_number, flow_table_name, self._year_data)
        res_flow_day_use = self.calculation_each_use(sql_flow_day, sql_flow_last_month)      # {'2021-05-24': 10.0, '2021-05-25': 1.0, '2021-05-26': 4.0, '2021-05-27': 5.0, '2021-05-28': 4.0, ...}
        for k in sorted(res_flow_day_use):
            flow_time_day.append(k)
            if str(res_flow_day_use[k]) == "0.0":
                flow_day_use.append(None)
            else:
                flow_day_use.append('%.2f' % res_flow_day_use[k])


        # ②当年流量趋势（t）（按月划分）
        flow_month_time = []
        flow_month_num = []
        sql_flow_month = "SELECT DATE_FORMAT(times, '%%Y-%%m') AS t, MAX(%s) AS num FROM %s WHERE times>=\'%s\' GROUP BY t;" % (flow_serial_number, flow_table_name, self._year_data)
        sql_flow_last_year = "SELECT times, %s AS num FROM %s WHERE times<\'%s\' ORDER BY times DESC LIMIT 1;" % (flow_serial_number, flow_table_name, self._year_data)
        res_flow_month = self.calculation_each_use(sql_flow_month, sql_flow_last_year)
        for i in range(1, 13):
            if len(str(i)) == 2:
                month_time = self._year_data + "-" + str(i)
            else:
                month_time = self._year_data + "-0" + str(i)
            if month_time not in res_flow_month.keys():
                if month_time <= self._month_data:
                    res_flow_month[month_time] = 0
                if month_time > self._month_data:
                    res_flow_month[month_time] = 0
        for k in sorted(res_flow_month.keys()):
            flow_month_time.append(k)
            ll_num = str("%.2f" % res_flow_month[k])
            if ll_num == "0.00":
                flow_month_num.append(None)
            else:
                flow_month_num.append(ll_num)


        for i in range(calendar.monthrange(int(self._month_data[:4]), int(self._month_data[-2:]))[1] + 1)[1:]:
            if len(str(i)) == 2:
                each_day = self._month_data + '-' + str(i)
            else:
                each_day = self._month_data + '-0' + str(i)
            if each_day not in ele_time_day:
                ele_time_day.append(each_day)
                ele_day_use.append('null')




        return_dict = {
            "ele_time_day": ele_time_day, "ele_day_use": ele_day_use,
            "ele_time_minute": ele_time_minute, "ele_today_ues": ele_today_ues, "ele_yesterday_ues": ele_yesterday_ues,
            "flow_time_day": flow_time_day, "flow_day_use": flow_day_use,
            "flow_month_time": flow_month_time, "flow_month_num": flow_month_num,
            "ele_year_all": ele_year_all
        }

        return return_dict




    def calculation_each_use(self, sql, sql_before_yesterday):
        """
        计算各设备每小时 / 每天的用电量
        :param sql:
        :return:
        """
        dicts = {}
        group_time = []         # 分组后的时间
        group_use = []          # 分组时间内的用量
        for each in self._operate_mysql.execute_sql(sql):
            group_time.append(each['t'])
            group_use.append(each['num'])


        if len(group_time) > 0:
            dicts[group_time[0]] = group_use[0]
            if len(group_time) >= 2:
                for i in range(len(group_time) - 1):
                    each_time = group_time[i + 1]
                    each_use = group_use[i + 1] - group_use[i]  # 每小时的实际用电
                    if each_use < 0:
                        each_use = 0
                    dicts[each_time] = each_use
            before_use = self._operate_mysql.execute_sql(sql_before_yesterday)
            if len(dicts) > 0 and len(before_use) > 0:
                before_time = min(dicts.keys())
                dicts[before_time] = dicts[before_time] - before_use[0]['num']

        return dicts



    def group_today_yesterday(self, datas):
        """
        将昨天和今天的用电量/累计流量分组
        :param datas:字典，关键字是时间（昨天和今天的时间，按照小时划分），值是每小时的实际用量（电量或流量）
        :return:
        """
        # 昨天和今天的趋势分组
        today_time = []         # 当日的时刻（按小时划分）
        today_use = []          # 当日每个时刻的用量
        yesterday_time = []     # 昨日的时刻（按小时划分）
        yesterday_use = []      # 昨日每个时刻的用量
        for k in sorted(datas):
            day_part = k[:10]           # 2021-05-26
            hour_part = k[-2:] + ":00"  # 19:00
            use_value = '%.1f' % datas[k]
            if day_part == self._yesterday_date:
                yesterday_time.append(hour_part)
                yesterday_use.append(use_value)
            elif day_part == self._today_date:
                today_time.append(hour_part)
                today_use.append(use_value)

        # 通过数据帧整合昨日和今日的用量
        df_today_use = pandas.DataFrame({self._today_date: today_use}, index=today_time)
        df_yesterday_use = pandas.DataFrame({self._yesterday_date: yesterday_use}, index=yesterday_time)

        df = (pandas.concat([df_yesterday_use, df_today_use], axis=1)).sort_index()       # 合并，并按index排序
        df = df.where(df.notnull(), 'null')                        # 将nan替换为'null'
        time_minute = df.index.tolist()
        today_ues = df[self._today_date].tolist()
        yesterday_ues = df[self._yesterday_date].tolist()

        return time_minute, today_ues, yesterday_ues
