#!/usr/bin/env python
# encoding: utf-8
"""
@CreateTime: 2022/05/06 10:35
@Author: lxc
@LastEditTime: 
@Desctiption: 返回平台所有类型的报警信息
"""

from abstract_api import AbstractApi
from event_storage import EventStorage
import datetime


class GetAlarmInfo(AbstractApi):

    def operation(self, request):
        self.operate_mysql = EventStorage()
        all_alarm_dict = {}

        # 水下设备清洗的提示
        underwater_equip_alarm = self.get_underwater_equip_alarm()
        all_alarm_dict["水下设备清洗提示报警"] = underwater_equip_alarm

        # 开关量综合报警
        moxa_e1210_alarm = self.get_moxa_e1210_alarm()
        all_alarm_dict["开关量综合报警提示"] = moxa_e1210_alarm


        return all_alarm_dict



    def get_underwater_equip_alarm(self):
        """水下设备清洗的提示"""
        underwater_equip_alarm_sql = "SELECT device_name, this_clean_time, clean_cycle, next_clean_time FROM underwater_equipment_clean WHERE function_status=1;"
        underwater_equip_alarm_sql_res = self.operate_mysql.execute_sql(underwater_equip_alarm_sql)
        result = []
        if len(underwater_equip_alarm_sql_res) > 0:
            now_time = datetime.datetime.now()
            for each in underwater_equip_alarm_sql_res:
                if now_time >= each["next_clean_time"]:
                    result.append(each)

        return result



    def get_moxa_e1210_alarm(self):
        """开关量综合报警"""
        io_point_name_dict = {}     # {'c365': '主配电板绝缘低', 'c366': '污水处理装置供电运行', 'c367': '污水处理装置综合报警', ...}
        select_io_point_name = "SELECT serial_number, io_point_name FROM `data_point_tbl` WHERE `station_name` = 'moxa_e1210' AND `io_point_name` IS NOT NULL;"
        io_point_name_res = self.operate_mysql.execute_sql(select_io_point_name)
        for each in io_point_name_res:
            io_point_name_dict["c" + str(each["serial_number"])] = each["io_point_name"]

        return_alarm_list = []
        select_sql = "SELECT CAST(create_time AS CHAR) AS create_time, CAST(update_time AS CHAR) AS update_time, serial_number FROM alarm_moxa_e1210 WHERE is_cancel=0;"
        alarm_datas = self.operate_mysql.execute_sql(select_sql)
        if alarm_datas:
            for each in alarm_datas:
                each_alarm = {
                    "alarm_create_time": each["create_time"],
                    "alarm_update_time": each["update_time"],
                    "io_point_name": io_point_name_dict[each["serial_number"]]
                }
                return_alarm_list.append(each_alarm)

        return return_alarm_list

