#!/usr/bin/env python
# encoding: utf-8
"""
@time: 2021/5/31 15:52
@desc: 获取历史数据，海水温度、海水深度、海水盐度、PH值、叶绿素等的历史数据
"""

from abstract_api import AbstractApi
from event_storage import EventStorage
from datetime import datetime

class GetWeatherHistoryData(AbstractApi):

    def operation(self, request):
        operate_mysql = EventStorage()
        serial_number = request['serial_number']
        begin_time = request['timeBegin']
        end_time = request['timeEnd']

        times = []
        value = []
        # 查该序列号对应的表名
        sql_table_name = "SELECT device_name FROM data_point_tbl WHERE serial_number=%s" % (serial_number.replace('c', ''))
        res_table_name = operate_mysql.execute_sql(sql_table_name)
        table_name = "table_" + res_table_name[0]['device_name']
        # 查数据(时间跨度超过7天，每分钟取一条数据)
        time_length = (datetime.strptime(end_time, '%Y-%m-%d %H:%M:%S')-datetime.strptime(begin_time, '%Y-%m-%d %H:%M:%S')).days
        if -7 < time_length < 7:
            sql_data = "SELECT CAST(times AS CHAR) as t, %s FROM %s WHERE times BETWEEN \'%s\' and \'%s\' ORDER BY times;" % (serial_number, table_name, begin_time, end_time)
        else:
            sql_data = f"SELECT (DATE_FORMAT(times, '%Y-%m-%d %H:%i')) AS t,  AVG({serial_number}) AS {serial_number} FROM {table_name} WHERE times BETWEEN \'{begin_time}\' and \'{end_time}\' GROUP BY t;"
        res_data = operate_mysql.execute_sql(sql_data)

        for each_data in res_data:
            data = each_data[serial_number]
            if data is None:
                pass
            else:
                times.append(each_data['t'])
                value.append(round(data, 3))

        history_data = {
            "times": times,
            "value": value
        }

        return history_data
