import json
import queue
import time
import threading
import datetime
import ast
from connector import Connector
from event_storage import EventStorage
from libs.hls1_1_0 import MelsecA1ENet
#from libs.hls import MelsecA1ENet
from log import OutPutLog

class MelsecPlcA1EConnector(Connector, threading.Thread):
    __master = 0
    _disConnectTime = 0

    def __init__(self, name, config, converter):
        super().__init__()
        self._log = OutPutLog()
        self.__master = None
        self.__stopped = False
        self.__connected = False
        self._ip = config['ip']  # ip
        self._port = config['port']  # 端口
        self.__save_frequency = config['save_frequency']  # 数据存储时间间隔
        self.setDaemon(True)
        self.setName(name)
        self.__converter = converter
        self.__storager = EventStorage()
        self.__command_queue = queue.Queue(1)
        self.__last_save_time = 0
        self.__data_point_config = self.__storager.get_station_info(name)
        self.__command = self.__storager.get_command_info(name)


    def open(self):
        self.__stopped = False
        self.start()

    def run(self):
        self.__connect()
        self.__connected = True
        command_list = json.loads(self.__command[0]['command'])
        while True:
            time.sleep(0.1)
            try:
                self.command_polling(command_list, resend_times=5)
            except Exception as e:
                print(e)
            # print('runing...')
            if self.__stopped:
                break

    def __connect(self):
        try:
            self.__master = MelsecA1ENet(self._ip, self._port)
            if self.__master.ConnectServer().IsSuccess == False:
                self.__connected = False
                print("MelsecPlcA1EConnector connect falied：", self._ip, self._port)
            else:
                self.__connected = True
                print("MelsecPlcA1EConnector connect success：", self._ip, self._port)
        except Exception as e:
            self._log.error(e)

    def __reconnect(self):
        pass

    def close(self):
        pass

    def get_name(self):
        return self.name

    def is_connected(self):
        return self.__connected

    def send_command(self, data):
        try:
            for command in data:
                try:
                    self.__command_queue.put_nowait(command)
                    return "success"
                except queue.Full:
                    return "queue.Full"           
        except Exception as e:
            print(e)
            return "send_command error"+str(e)

    def command_polling(self, command_list, resend_times=None):
        #msg = str(time.strftime("%Y-%m-%d %H:%M:%S")
        for i in range(len(command_list)):
            command_item = command_list[i]
            # print("self.__command_queue.qsize()123221 = ", self.__command_queue.qsize())
            if not self.__command_queue.empty():
                # print("self.__command_queue.qsize() before write = ", self.__command_queue.qsize())
                write_command = self.__command_queue.get()  # 写命令来自数组
                # print("self.__command_queue.qsize() after write = ", self.__command_queue.qsize())                
                try:
                    self.writeData(write_command['addr_data'], write_command['data_type'], write_command['data'])
                    print("send:"+ str(time.time()))
                    #self._log.info('success'+str(res))
                except Exception as e:
                    print("A1EConverter,write[ERROR]:" + str(e))
                    self._log.error("A1EConverter,write[ERROR]:" + str(e))
                '''
                if not res:
                    # 根据（轮询预计用时，剩余可用时间）计算重发次数
                    sent_times = 1
                    while sent_times < resend_times:
                        res = self.exec_command(write_command)
                        sent_times += 1
                        if res:
                            break
                '''
            else:
                # 格式化数据
                result = {}
                start_addr = command_item['start_addr']
                length = command_item['length']
                db_number = start_addr[:1]              
                result[db_number] = self.read_batch_data(start_addr, length)
                format_data = self.__converter.convert(self.__data_point_config, result, self.__master)
                if format_data:
                    if format_data != "error" and format_data != 'pass':
                         # 往redis存储数据
                        self.__storager.real_time_data_storage(format_data)
                        self._log.info('success -- 实时数据存入redis！')
    '''
    def read_batch_data(self, addr, length):
        try:
            self.__master.ConnectServer().IsSuccess)
            result = self.__master.Read(addr, length)
            if result.IsSuccess:
                print("success____________________________________________")
            else:
                print("falied_______________________________________  " + result.Message) 
            #datas = self.__master.Read(addr, length).Content
            #print(datas)
            return None
            #return datas
        except Exception as e:
            print("A1EConverter,read[ERROR]:" + str(e))
            return None
    '''
            
    def read_batch_data(self, addr, length):
        try:
            result = self.__master.Read(addr, length)
            if result.IsSuccess:
                datas = result.Content
                return datas
            else:
                if self.__master.ConnectServer().IsSuccess == False:
                    self.__connected = False
                else:
                    self.__connected = True
                return None
        except Exception as e:
            print("A1EConverter,read[ERROR]:" + str(e))
            return None            

    def writeData(self, address, data_type, data):
        data_addr = self.address_transform(address)
        if data_type == "BOOL":
            data = int(data)
            self.__master.WriteBool(data_addr, data)

    def address_transform(self, addr):
        return addr
