"""
@Date  :2021/5/21/00219:10:57
@Desc  :
"""
import json
import time
import threading
import socket
import queue
from log import OutPutLog
from connector import Connector
from event_storage import EventStorage


class TcpConnector(Connector, threading.Thread):
    def __init__(self, name, config, converter):
        super().__init__()
        self._log = OutPutLog()
        self.__sock = None
        self.__connected = False
        self.__stopped = False
        self.__size = 1024
        self.__ip = config['ip']
        self.__port = config['port']
        self.__converter = converter
        self.__storager = EventStorage()
        self.__save_frequency = config['save_frequency']
        self.__command_queue = queue.Queue(50)
        self.setDaemon(True)
        self.setName(name)
        self.__last_seve_time = 0
        self.__data_point_config = self.__storager.get_station_info(name)

    def open(self):
        self.__stopped = False
        self.start()

    def run(self):
        self.__connect()
        self.__connected = True
        while True:
            time.sleep(1)
            self.command_polling()
            if self.__stopped:
                break

    # socket
    def __connect(self):
        if self.__sock:
            self.close()
        self.__sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.__sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)  # ñصַͶ˿
        self.__sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)  # ڿͻ˿ά
        # self.__sock.settimeout(180)  # óʱʱ3mins
        self.__sock.settimeout(10)  # óʱʱ3mins
        try:
            self.__sock.connect((self.__ip, self.__port))
            print(self.__sock)
            self.__connected = True
            print("connect success!")
        except Exception as e:
            print("connect failed")
            self.__connected = False
            self.__reconnect()

    def __reconnect(self):
        while True:
            try:
                self.__sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                self.__sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                self.__sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)  # ڿͻ˿ά
                # self.__sock.settimeout(180)  # óʱʱ3mins
                self.__sock.settimeout(10)  # óʱʱ3mins
                self.__sock.connect((self.__ip, self.__port))
                print(self.__sock)
                self.__connected = True
                print("reconnect success")
                break
            except Exception as e:
                self.__connected = False
                print("reconnect failed")
                time.sleep(5)

    def close(self):
        """Close the connection with the TCP Slave"""
        if self.__sock:
            self.__sock.close()
            self.__stopped = True
            self.__sock = None
            self.__connected = False
            return None

    def get_name(self):
        return self.name

    def is_connected(self):
        return self.__connected

    def send_command(self, data):
        if self.__sock:
            # print("**************************", self.__ip, type(data),data)
            try:
                if isinstance(data, str):
                    restult_command = self.__sock.send(data.encode(encoding='utf-8'))
                elif isinstance(data, list):
                    restult_command = self.__sock.send(data[1])
                    if data[0] == "read_status":
                        return_command = ""
                        while len(return_command) != 15:
                            return_command = self.__sock.recv(15)
                        return return_command
                # restult_command = self.__sock.send(data.encode(encoding='utf-8'))
                # print("[tcp_connector]send_command: ", restult_command)
            except Exception as e:
                print(self.__ip, "[tcp_connector]send_command error: ", e)

    def command_polling(self):
        if self.__connected:
            try:
                time.sleep(0.2)
                data = self.__sock.recv(self.__size)
                data = self.__converter.convert(self.__data_point_config, data)
                print(self.__sock)
                if data == b'':
                    self.__reconnect()
                print(data)
                if data:
                    if data != "error" and data != 'pass':
                        self.__storager.real_time_data_storage(data)
            except Exception as e:
                time.sleep(5)
                self.__reconnect()
        else:
            self.__reconnect()

