from converter import Converter
import struct
import binascii
from log import OutPutLog
from event_storage import EventStorage


class ModbusConverter(Converter):
    list = []

    def __init__(self):
        self._log = OutPutLog()
        self._storage = EventStorage()


    def convert(self, config, data):
        """
        :param config: [{'id': 27, 'station_name': 'modbustcptest' ...}, {'id': 28, 'station_name': 'modbustcptest' ...} ..]
        :param data: [0, 1, 2, 3, 4, 5, 6, 7, 65535, 0, 65535]
        :return: {'c201': 0, 'c202': 1, 'c203': 2, 'c204': 3, 'c205': 4, 'c206': 5 ...}
        """
        if data:
            format_data_dict = {}  # 列表数据转换字典数字
            try:
                for index in config:
                    # addr_type : 'D' 或 'X';
                    # addr_list: [10] 或 [10, 5]
                    addr_type, addr_list = addr_parsing(index['address'])
                    register_addr = addr_list[0]  # 数据地址  example:'X10.5' -> 10
                    if register_addr in data:
                        name = 'c' + str(index['serial_number'])
                        if addr_type == "R":
                            # 从redis获取数据
                            real_data = self._storage.get_real_data([name])
                            if real_data:
                                return_data = real_data[name]
                            else:
                                return_data = None
                        elif addr_type == 'X':
                            bit_offset_addr = addr_list[1]  # 位偏移地址  example:'X10.5' -> 5
                            if index['data_type'] == "BOOL":
                                data_bin = bin(65536 | data[register_addr])  # 和65536做或运算，为了留开头0
                                data_bin = list(data_bin[3:][::-1])  # 去除开头字符 0  b  1并反转字符串
                                data_bin = list(map(int, data_bin))  # 字符列表转整型列表
                                if index['modbus_mode'] == "BA":
                                    data_bin = data_bin[8:16] + data_bin[0:8]
                                if index['negation'] == 1:
                                    return_data = int(bool(1-data_bin[bit_offset_addr])) #取反运算
                                else:
                                    return_data = data_bin[bit_offset_addr]
                            # print("data=", data, register_addr, return_data)
                        elif addr_type == 'D':
                            if index['data_type'] == "FLOAT16":
                                return_data = data[register_addr]
                            elif index['data_type'] == "INT16":
                                if index['negation'] == 1:
                                    return_data = int(bool(1-data[register_addr])) #取反运算
                                else:
                                    if data[register_addr] > 32767:
                                        return_data = data[register_addr] - 65536
                                    else:
                                        return_data = data[register_addr]

                            elif index['data_type'] == "INT32":
                                return_data_H = data[register_addr]
                                return_data_L = data[register_addr + 1]
                                if index['modbus_mode'] == "CDAB":
                                    return_data = data[register_addr] * 65536 + data[register_addr+1]
                                else:
                                    return_data = data[register_addr + 1] * 65536 + data[register_addr]
                            elif index['data_type'] == "FLOAT32":
                                t1 = hex(data[register_addr])[2:]
                                t2 = hex(data[register_addr+1])[2:]
                                if len(t1) < 4:
                                    t1 = (4-len(t1)) * "0" + t1
                                if len(t2) < 4:
                                    t2 = (4 - len(t2)) * "0" + t2
                                return_data = struct.unpack('>f', binascii.unhexlify((t1+t2).replace(' ', '')))[0]
                            elif index['data_type'] == "BELZ_FLOAT32":
                                llj_data = []
                                for x in range(0, 6, 2):
                                    t1 = hex(data[register_addr+x])[2:]
                                    t2 = hex(data[register_addr+x+1])[2:]
                                    if len(t1) < 4:
                                        t1 = (4 - len(t1)) * "0" + t1
                                    if len(t2) < 4:
                                        t2 = (4 - len(t2)) * "0" + t2
                                    t = struct.unpack('>f', binascii.unhexlify((t1 + t2).replace(' ', '')))[0]
                                    llj_data.append(t)
                                return_data = llj_data[0] * 10**6 + llj_data[1] + llj_data[2]/10**6        # 总累计量
                            if index['divisor'] is not None:
                                return_data = return_data / index['divisor']
                            if isinstance(return_data, float):
                                return_data = round(return_data, 2)
                        # name = 'c' + str(index['serial_number'])
                        #################################
                        if str(index['serial_number']) == index['device_id'] or index['device_id'] == '0':
                            # print(index['serial_number'], return_data)
                            if return_data is not None:
                                format_data_dict[name] = return_data
                        else:
                            com_name = 'c' + index['device_id']
                            list_com_name = [com_name]
                            status_list = self._storage.get_real_data(list_com_name)
                            status = status_list[com_name]
                            # print(index['serial_number'], status, return_data)
                            if status == '0':
                                format_data_dict[name] = return_data
                        ##################################
                        # format_data_dict[name] = return_data
                print("format_data_dict=", format_data_dict)
                return format_data_dict
            except Exception as e:
                print(e)
                self._log.error(e)
                return "error"


def addr_parsing(addr_smash):
    """
    :param addr_smash: [X15.1] or [D15]
    :return: X, [15, 1] or D [15]
    """
    # addr_smash_1 = 'D2'
    # addr_smash = list(addr_smash)  # 拆分字符串为list ['X', '6', '.', '2']
    addr_type = addr_smash[0]  # 地址类型 : 'D' 或 'X'
    addr_smash = addr_smash[1:]  # 地址部分: '10' 或 '10.5'
    addr_list = list(map(int, addr_smash.split(".")))  # 用“.”分割字符串转换为整型存入列表
    return addr_type, addr_list
