"""
@Date  :2021/5/24/10:20
@Desc  :
"""



import json
import threading

from event_storage import EventStorage
import time
from log import OutPutLog


class HistoricalDataStorage:
    def __init__(self, read_write):
        self._storage = EventStorage()
        self._log = OutPutLog()
        self._read_write = read_write


    # 历史存储主函数
    def run(self):
        self._log.info('[HistoricalDataStorage] - Historical data storage module is running!')
        station_info = self._storage.hardDiskStorage.get_connectors(self._read_write)  # 获取所有站点信息
        all_devices = []
        for item in station_info:
            station_name = item['station_name']  # 站点名称
            connector_config = json.loads(item['connector_config'])     # 加载json格式connector_config参数
            save_frequency = connector_config['save_frequency']         # 获取存储频率
            devices_each_station = self._storage.hardDiskStorage.get_device_name_by_station_name(station_name)  # 根据站点名称获取设备列表

            for i in devices_each_station:
                temp_dict = {}

                # 获取每个设备所有点的serial_number,转换为键列表
                device_name = i['device_name']
                data_point_each_device = self._storage.hardDiskStorage.get_data_point_by_device_name(
                    device_name)  # 根据设备名称获取设备点表

                serial_number_list = []
                for items in data_point_each_device:
                    serial_number = 'c' + str(items['serial_number'])
                    serial_number_list.append(serial_number)

                temp_dict['device_name'] = device_name
                temp_dict['save_frequency'] = save_frequency
                temp_dict['serial_number_list'] = serial_number_list
                temp_dict['last_save_time'] = 0
                all_devices.append(temp_dict)

        while 1:
            time.sleep(0.05)     # 修改time.sleep(0.2)
            for item in all_devices:
                save_frequency = item['save_frequency']
                last_save_time = item['last_save_time']
                now_time = int(time.time())  # 优化数据丢失的情况
                if now_time - last_save_time >= save_frequency:
                    item['last_save_time'] = now_time
                    save_time = int(now_time) - int(now_time) % save_frequency
                    save_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(save_time))

                    serial_number_list = item['serial_number_list']
                    real_time_data = self._storage.memoryStorage.get_value(serial_number_list)  # 根据键列表查询实时数据库
                    flag = False  # 是否允许存储标志位
                    for key in real_time_data:
                        # 值全部为空，不允许存储
                        if real_time_data[key]:
                            flag = True
                            break
                    if flag and len(serial_number_list) >= 1:
                        for key in real_time_data:  # redis值为None的
                            if real_time_data[key] is None:  # redis数据库未存储此值
                                real_time_data[key] = 'null'
                            if real_time_data[key] == '':  # redis存储的为空值
                                real_time_data[key] = 'null'
                        table_name = "table_" + str(item['device_name'])  # 根据站名计算表名
                        # self._log.debug("[HistoricalDataStorage] - " + repr(table_name) + '<-' + repr(real_time_data))
                        self._storage.hardDiskStorage.insert_column_many(table_name, save_time, real_time_data)
                        #self._log.info(repr(table_name) + '<-' + repr(real_time_data))
