#!/usr/bin/env python
# encoding: utf-8
"""
@CreateTime: 2022/06/16 17:20
@Author: lxc
@LastEditTime: 
@Desctiption:判断接入集控系统的所有设备的参数的实时值是否异常，并记录
"""

import time
from event_storage import EventStorage
from log import OutPutLog
from datetime import datetime

class AlarmRecord:
    def __init__(self):
        self._storage = EventStorage()
        self._log = OutPutLog()


    def run(self):
        run_time = 0
        while True:
            this_time = time.time()
            if this_time - run_time > 10:
                run_time = this_time
                self.now_time = datetime.now()
                self.deal_alarm()



    def deal_alarm(self):

        # 查询已经存在的报警参数信息
        alarm_point_dict = self.get_alarm_tbl()
        # 查看点表中，所有需要判断是否需要报警的点
        point_list, point_dict = self.get_buzzer_alarm_point()
        # 读取实时数据
        last_data_dict = self.get_real_data(point_list)


        for serial_number, value in last_data_dict.items():
            try:
                # 区分参数类型：Switch、Analog、device_clean
                if serial_number.startswith("c"):
                    signal_type = point_dict[serial_number][0]
                else:
                    signal_type = "device_clean"

                # 判断该参数的值是否正常
                alarm_status = 0      # 默认该参数的数值正常。0:正常，1：异常
                if value is None:
                    # alarm_status = 1
                    pass
                else:
                    if signal_type == "Switch":
                        alarm_status = int(value)
                    elif signal_type == "Analog":
                        alarm_low_limit = point_dict[serial_number][1]
                        alarm_up_limit = point_dict[serial_number][2]
                        if alarm_low_limit is not None and alarm_up_limit is not None:
                            alarm_status = 0 if float(alarm_low_limit) <= float(value) <= float(alarm_up_limit) else 1
                    elif signal_type == "device_clean" and value < self.now_time:
                        alarm_status = 1
                        value = None

                    # print(serial_number, alarm_status)


                # 更新到数据库
                if serial_number in alarm_point_dict.keys():
                    # 该参数的报警已经存在，更新alarm_tbl表的update_time、alarm_point_status字段
                    alarm_id = alarm_point_dict[serial_number]
                    if alarm_status == 1:
                        update_sql = f"UPDATE `alarm_tbl` SET `update_time`=\'{self.now_time}\', `alarm_point_status`=0 WHERE `id`={alarm_id};"
                    elif alarm_status == 0:
                        update_sql = f"UPDATE `alarm_tbl` SET `update_time`=\'{self.now_time}\', `alarm_point_status`=1, `is_cancel`=1 WHERE `id`={alarm_id};"

                    update_result = self._storage.execute_update_sql(update_sql)
                    # print(update_result, update_sql)
                    # self._log.info([update_result, update_sql])

                elif alarm_status == 1:
                    # 产生新的报警点，alarm_tbl表插入一条新的报警数据
                    insert_sql = f"INSERT INTO `alarm_tbl` SET `point_name`=\'{serial_number}\',`data`={value}, `signal_type`=\'{signal_type}\';"
                    insert_sql = insert_sql.replace("None", "NULL")
                    insert_result = self._storage.execute_update_sql(insert_sql)
                    # print(insert_result, insert_sql)
                    # self._log.info([insert_result, insert_sql])

            except Exception as e:
                print(f"[AlarmAll]: {e}")
                self._log.error(e)


    def get_alarm_tbl(self):
        """
        查询已经存在的报警信息
        :return: {'c14': 2, 'c12': 1, 'c369': 9, 'c373': 12, 'c375': 13, ...}
        """
        alarm_sql = f"SELECT id,point_name FROM alarm_tbl WHERE alarm_point_status=0;"
        alarm_info = self._storage.execute_sql(alarm_sql)
        alarm_point_dict = {}
        for each_alarm in alarm_info:
            alarm_point_dict[each_alarm["point_name"]] = each_alarm["id"]
        return alarm_point_dict


    def get_buzzer_alarm_point(self):
        """
        # 查看点表和水下设备清洗配置表中，所有需要判断是否需要报警的点
        :return:(['c365', 'c366', ...], {'c365': ['Switch', 0.0, 1.0], 'c366': ['Switch', 0.0, 1.0], ...})
        """
        point_sql = "SELECT serial_number,alarm_low_limit,alarm_up_limit,signal_type FROM data_point_tbl WHERE buzzer_alarm=1"
        point_info = self._storage.execute_sql(point_sql)
        point_list = []
        point_dict = {}
        for index in point_info:
            point_list.append('c' + str(index['serial_number']))
            point_dict['c' + str(index['serial_number'])] = [index['signal_type'], index['alarm_low_limit'], index['alarm_up_limit']]

        return point_list, point_dict


    def get_real_data(self, point_list):

        last_data_dict = self._storage.memoryStorage.get_value(point_list)
        # last_data_dict = {'c1': None, 'c2': 25.5, 'c3': 70, 'c4': 20, 'c5': 100, 'c6': 10, 'c365': 1, 'c366': 0, 'c367': 0, 'c368': 0, 'c369': 1, 'c370': 0, 'c371': 0, 'c372': 0, 'c373': 0, 'c374': 0, 'c375': 0, 'c376': 0, 'c377': 0, 'c416': 0, 'c417': 0}

        # 添加水下设备清洗配置信息
        get_underwater_equip_sql = f"SELECT device_name, next_clean_time FROM underwater_equipment_clean WHERE function_status=1;"
        get_underwater_equip_info = self._storage.execute_sql(get_underwater_equip_sql)
        for index in get_underwater_equip_info:
            last_data_dict[index["device_name"]] = index["next_clean_time"]

        return last_data_dict
