#!/usr/bin/env python
# encoding: utf-8
"""
@CreateTime: 2021/12/8 14:57
@Author: lxc
@LastEditTime: 
@Desctiption: 蜂鸣器的连接器，控制蜂鸣器的开关
"""

import json
import threading
import queue
from connector import Connector
from event_storage import EventStorage
from log import OutPutLog
import time
import datetime
from modbus_tk import modbus_rtu_over_tcp


class ModbusRtuOverTcpNormalConnector(Connector, threading.Thread):

    def __init__(self, name, config, converter):
        super().__init__()
        self._log = OutPutLog()
        self._master = None
        self.__stopped = False
        self._connected = False
        self._ip = config['ip']  # ip
        self._port = config['port']  # 端口
        self._save_frequency = config['save_frequency']  # 数据存储时间间隔
        self._set_timeout = int(config['set_timeout'])	#读取超时时间设置
        self.setDaemon(True)
        self.setName(name)
        self.__converter = converter
        self.__storager = EventStorage()
        self.__command_queue = queue.Queue(500)
        self.__data_point_config = self.__storager.get_station_info(name)
        self._name = name
        self.run_time = 0
        self._command = self.__storager.get_command_info(name)


    def open(self):
        self.__stopped = False
        self.start()


    def run(self):
        self._connect()
        self._connected = True
        if isinstance(self._command, list):
            command_list = json.loads(self._command[0]['command'])
            # print("----------------------", self._ip, self._port, type(command_list), command_list)
            while True:
                time.sleep(1)
                self.command_polling(command_list, resend_times=5)
                if self.__stopped:
                    break

    def _connect(self):
        try:
            self._master = modbus_rtu_over_tcp.RtuOverTcpMaster(host=self._ip, port=self._port)
            print(f"{self._ip}:{self._port} connect success!")
        except Exception as e:
            print(f"{self._ip}:{self._port} connect failed!")
            self._log.error(f"{self._ip}:{self._port} connect failed! error={e}")
            self._connected = False
            self._reconnect()


    def _reconnect(self):
        while True:
            try:
                self._master = modbus_rtu_over_tcp.RtuOverTcpMaster(host=self._ip, port=self._port)
                print('client start connect to host/port:{}'.format(self._port))
                break
            except ConnectionRefusedError:
                print('modbus server refused or not started, reconnect to server in 2s .... host/port:{}'.format(self._port))
                time.sleep(2)
            except Exception as e:
                print('do connect error:{}'.format(str(e)))
                time.sleep(2)


    def close(self):
        pass

    def get_name(self):
        return self.name

    def is_connected(self):
        return self._connected

    def send_command(self, command):
        # command = {'device_id': 1, 'start_addr': 0, 'output_value': [0, 0, 0, 0], 'function_code': 15}
        # print(f"[send_command] {command}")
        try:
            if isinstance(command, dict):
                result = self.exec_command(command)
            elif isinstance(command, list):
                for each in command:
                    result = self.exec_command(each)
        except Exception as e:
            print(f"[ModbusTcpConnector][send_command] error: {e}")
            result = False

        # print(f"[send_command][result] {result}")
        return result



    def exec_command(self, command):
        """
        0x01: 读线圈寄存器
        0x02: 读离散输入寄存器
        0x03: 读保持寄存器
        0x04: 读输入寄存器
        0x05: 写单个线圈寄存器
        0x06: 写单个保持寄存器
        0x0f: 写多个线圈寄存器
        0x10: 写多个保持寄存器
        #supported modbus functions
        READ_COILS = 1
        READ_DISCRETE_INPUTS = 2
        READ_HOLDING_REGISTERS = 3
        READ_INPUT_REGISTERS = 4
        WRITE_SINGLE_COIL = 5
        WRITE_SINGLE_REGISTER = 6
        READ_EXCEPTION_STATUS = 7
        DIAGNOSTIC = 8
        REPORT_SLAVE_ID = 17
        WRITE_MULTIPLE_COILS = 15
        WRITE_MULTIPLE_REGISTERS = 16
        READ_WRITE_MULTIPLE_REGISTERS = 23
        DEVICE_INFO = 43
        """
        time.sleep(0.1)
        if isinstance(command, str):
            command = json.loads(command)
        device_id = int(command['device_id'])
        function_code = int(command['function_code'])
        start_addr = int(command['start_addr'])

        if function_code in (1, 2, 3, 4):
            # 读寄存器
            length = int(command['length'])
            try:
                self._master.set_timeout(self._set_timeout)
                self._master.set_verbose(True)
                # print(self._ip, self._port, device_id, function_code, start_addr, length)
                receive_data = self._master.execute(device_id, function_code, start_addr, length)
                datadict = {}
                for i in range(len(receive_data)):
                    addr = start_addr+i
                    datadict[addr] = receive_data[i]
                result = [device_id, datadict]

                # print(self._ip, self._port,device_id, result)

            except Exception as e:
                print(self._ip, self._port, device_id, start_addr, length, "error = ", e)
                self._log.error(e)
                result = False
            return result

        elif function_code in (5, 6, 15, 16):
            # 写寄存器
            output_value = command['output_value']
            try:
                self._master.set_timeout(1.0)
                self._master.set_verbose(True)
                # print("-----------------", self._ip, self._port, command)
                data = self._master.execute(device_id, function_code, start_addr, output_value=output_value)
                print("************************-----------------", self._ip, self._port, device_id,output_value, data)
                # print("data = ", data)
                # data = (0, 65280) or (0, 0)
                result = True
                if function_code == 5 and "res" in command.keys():
                    res = command["res"]
                    if start_addr == data[0] and res == data[1]:
                        pass
                    else:
                        result = False
                return result
            except Exception as e:
                self._log.error(e)
                print(f"[exec_command] e = {e}")
                return False
        else:
            self._log.debug(f'Unsupported function code.')



    def command_polling(self, command_list, resend_times=None):
        #msg = str(time.strftime("%Y-%m-%d %H:%M:%S"))
        for i in range(len(command_list)):
            command_item = command_list[i]
            if not self.__command_queue.empty():
                write_command = self.__command_queue.get()  # 写命令来自数组
                try:
                    res = self.exec_command(command=write_command)
                    print(res)
                except Exception as e:
                    print("mofbus_rtu,write[ERROR]:" + str(e))
                '''
                if not res:
                    # 根据（轮询预计用时，剩余可用时间）计算重发次数
                    sent_times = 1
                    while sent_times < resend_times:
                        res = self.exec_command(write_command)
                        sent_times += 1
                        if res:
                            break
                '''

            else:
                # 格式化数据
                result = self.exec_command(command=command_item)
                if result:
                    format_data = self.__converter.convert(self.__data_point_config, result)
                    #print(format_data)
                    if format_data:
                        if format_data != "error" and format_data != 'pass':
                            # 往redis存储数据
                            self.__storager.real_time_data_storage(format_data)
