#!/usr/bin/env python
# encoding: utf-8
"""
@CreateTime: 2022/07/03 08:50
@Author: lxc
@LastEditTime:
@Desctiption:
"""

import json
import queue
import time
import traceback

import threading
# from modbus_tk import modbus_tcp
from connector import Connector
from event_storage import EventStorage
import modbus_tk.modbus_tcp as modbus_tcp
from log import OutPutLog

class ModbusTcpConnector(Connector, threading.Thread):
    __master = 0
    _disConnectTime = 0

    def __init__(self, name, config, converter):
        super().__init__()
        self._log = OutPutLog()
        self.__master = None
        self.__stopped = False
        self.__connected = False
        self._ip = config['ip']  # ip
        self._port = config['port']  # 端口
        self.__save_frequency = config['save_frequency']  # 数据存储时间间隔
        self.setDaemon(True)
        self.setName(name)
        self.__converter = converter
        self.__storager = EventStorage()
        self.__command_queue = queue.Queue(500)
        self.__last_save_time = 0
        self.__data_point_config = self.__storager.get_station_info(name)
        self.__command = self.__storager.get_command_info(name)


    def open(self):
        self.__stopped = False
        self.start()

    def run(self):
        self.__connect()
        self.__connected = True
        command_list = json.loads(self.__command[0]['command'])
        while True:
            time.sleep(1)
            self.command_polling(command_list, resend_times=5)
            if self.__stopped:
                break

    def __connect(self):
        try:
            self.__master = modbus_tcp.TcpMaster(host=self._ip, port=self._port)
            print(self._ip, self._port, 'connect success!')

        except Exception as e:
            self._log.error(e)
            print(self._ip, self._port, 'connect failed!', e)


    def __reconnect(self):
        while True:
            try:
                # self.__master = modbus_tcp.TcpMaster(host=self._ip)
                self.__master = modbus_tcp.TcpMaster(host=self._ip, port=self._port)
                print('client start connect to host/port:{}'.format(self._port))
                break
            except ConnectionRefusedError:
                print(self._ip, self._port, 'reconnect success!')
                time.sleep(5)
            except Exception as e:
                print(self._ip, self._port, 'reconnect failed!', e)
                time.sleep(5)

    def close(self):
        pass

    def get_name(self):
        return self.name

    def is_connected(self):
        return self.__connected

    def send_command(self, data):
        try:
            for command in data:
                self.__command_queue.put(command)
            return "success"
        except Exception as e:
            print(e)
            return "send_command error"+str(e)

    def exec_command(self, command):
        """
        0x01: 读线圈寄存器
        0x02: 读离散输入寄存器
        0x03: 读保持寄存器
        0x04: 读输入寄存器
        0x05: 写单个线圈寄存器
        0x06: 写单个保持寄存器
        0x0f: 写多个线圈寄存器
        0x10: 写多个保持寄存器
        #supported modbus functions
        READ_COILS = 1
        READ_DISCRETE_INPUTS = 2
        READ_HOLDING_REGISTERS = 3
        READ_INPUT_REGISTERS = 4
        WRITE_SINGLE_COIL = 5
        WRITE_SINGLE_REGISTER = 6
        READ_EXCEPTION_STATUS = 7
        DIAGNOSTIC = 8
        REPORT_SLAVE_ID = 17
        WRITE_MULTIPLE_COILS = 15
        WRITE_MULTIPLE_REGISTERS = 16
        READ_WRITE_MULTIPLE_REGISTERS = 23
        DEVICE_INFO = 43
        """
        time.sleep(0.1)
        datadict = {}
        msg = str(time.strftime("%Y-%m-%d %H:%M:%S"))
        device_id = int(command['device_id'])
        function_code = int(command['function_code'])
        start_addr = int(command['start_addr'])
        start_addr_command = start_addr


        if function_code in (1, 2, 3, 4):
            # 读寄存器
            try:
                length = int(command['length'])
                self.__master.set_timeout(0.5)
                self.__master.set_verbose(True)
                res = ()
                num = 40
                if length > num:
                    count = length/num
                    while count > 0:
                        if count < 1:
                            le = length % num
                        else:
                            le = num
                        receive_data = self.__master.execute(device_id, function_code, start_addr_command, le)     # 203 3 4096 20
                        res = res+receive_data
                        start_addr_command = start_addr_command+num
                        count = count-1
                else:
                    le = length
                    # print("-----------------------",device_id, function_code, start_addr, le)
                    res = self.__master.execute(device_id, function_code, start_addr, le)  # 203 3 4096 20
                    # print("--------------------", res)
                for i in range(len(res)):
                    addr = start_addr+i
                    datadict[addr] = res[i]
                result = [device_id, datadict]
                # print("--------------------",self._ip, self._port, result)
                return result
            except Exception as e:
                self._log.error(e)
        elif function_code in (5, 6, 15, 16):
            # 写寄存器
            try:
                output_value = command['output_value']
                self.__master.set_timeout(1.0)
                self.__master.set_verbose(True)
                data = self.__master.execute(device_id, function_code, start_addr, output_value=output_value)
                print(data)
                return data
            except Exception as e:
                self._log.error(e)
        else:
            self._log.debug(f'Unsupported function code.')

    def command_polling(self, command_list, resend_times=None):
        #msg = str(time.strftime("%Y-%m-%d %H:%M:%S"))
        for i in range(len(command_list)):
            command_item = command_list[i]
            if not self.__command_queue.empty():
                write_command = self.__command_queue.get()  # 写命令来自数组
                try:
                    res = self.exec_command(command=write_command)
                    print(res)
                except Exception as e:
                    print("mofbus_rtu,write[ERROR]:" + str(e))
                '''
                if not res:
                    # 根据（轮询预计用时，剩余可用时间）计算重发次数
                    sent_times = 1
                    while sent_times < resend_times:
                        res = self.exec_command(write_command)
                        sent_times += 1
                        if res:
                            break
                '''
            else:
                # 格式化数据
                result = self.exec_command(command=command_item)
                format_data = self.__converter.convert(self.__data_point_config, result)
                #print(format_data)
                if format_data:
                    if format_data != "error" and format_data != 'pass':
                        # 往redis存储数据
                        self.__storager.real_time_data_storage(format_data)
