"""
@Date  :2021/5/21/00219:10:57
@Desc  :
"""
import json
import time
import threading
import socket
import queue
from log import OutPutLog
from connector import Connector
from event_storage import EventStorage
from datetime import datetime

class TcpConnector(Connector, threading.Thread):
    def __init__(self, name, config, converter):
        super().__init__()
        self._log = OutPutLog()
        self.__sock = None
        self.__connected = False
        self.__stopped = False
        self.__size = 1024
        self.__ip = config['ip']
        self.__port = config['port']
        self.__converter = converter
        self.__storager = EventStorage()
        self.__save_frequency = config['save_frequency']
        self.__command_queue = queue.Queue(50)
        self.setDaemon(True)
        self._name = name
        self.setName(name)
        self.__last_seve_time = 0
        self.__data_point_config = self.__storager.get_station_info(name)
        self.__command = self.__storager.get_command_info(name)
        # print(self.__ip, self.__command, len(self.__command))
        # print(self.__data_point_config)

    def open(self):
        self.__stopped = False
        self.start()

    def run(self):
        self.__connect()
        # self.__connected = True
        while True:
            time.sleep(1)
            if len(self.__command) > 0 and isinstance(self.__command, list):
                if self.__command[0]['command']:
                    command_list = json.loads(self.__command[0]['command'])
                    self.command_polling2(command_list)
            else:
                self.command_polling()
            if self.__stopped:
                break

    # socket
    def __connect(self):
        if self.__sock:
            self.close()
        self.__sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.__sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)  # ñصַͶ˿
        self.__sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)  # ڿͻ˿ά
        # self.__sock.settimeout(180)  # óʱʱ3mins
        self.__sock.settimeout(1)  # óʱʱ3mins
        try:
            self.__sock.connect((self.__ip, self.__port))
            # print(self.__sock)
            self.__connected = True

            print(datetime.now(), self.__ip, self.__port, "connect success!")
        except Exception as e:
            print(datetime.now(), self.__ip, self.__port, "connect failed", e)
            self.__connected = False
            self.__reconnect()

    def __reconnect(self):
        while True:
            try:
                self.__sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                self.__sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                self.__sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)  # ڿͻ˿ά
                # self.__sock.settimeout(180)  # óʱʱ3mins
                self.__sock.settimeout(1)  # óʱʱ3mins
                self.__sock.connect((self.__ip, self.__port))
                # print(self.__sock)
                self.__connected = True
                print(datetime.now(), self.__ip, self.__port, "reconnect success")
                break
            except Exception as e:
                self.__connected = False
                print(datetime.now(), self.__ip, self.__port, "reconnect failed", e)
                time.sleep(5)

    def close(self):
        """Close the connection with the TCP Slave"""
        if self.__sock:
            self.__sock.close()
            self.__stopped = True
            self.__sock = None
            self.__connected = False
            return None

    def get_name(self):
        return self.name

    def is_connected(self):
        return self.__connected

    def send_command(self, data):
        # bytes.fromhex(each_command)
        if self.__sock:
            # print("**************************", self.__ip, type(data),data)
            try:
                if isinstance(data, str):
                    self.__sock.send(bytes.fromhex(data))
                    return True
                elif isinstance(data, dict):
                    command = data["command"]
                    command = bytes.fromhex(command)
                    self.__sock.send(command)

                    if "size" in data.keys():
                        return_command = ""
                        size = data["size"]  # յָĳ
                        while len(return_command) != size:
                            return_command = self.__sock.recv(size)
                        return return_command

                    elif "res_command" in data.keys():
                        return_command = self.__sock.recv(len(command))
                        res_command = bytes.fromhex(data["res_command"])   # Ӧýյָ
                        if res_command == return_command:
                            return True
                        else:
                            return False
            except Exception as e:
                print(datetime.now(), self.__ip, "[tcp_connector]send_command error: ", e)
                self.__connected = False
                return False


    def command_polling(self):
        if self.__connected:
            try:
                time.sleep(0.2)
                data = self.__sock.recv(self.__size)
                data = self.__converter.convert(self.__data_point_config, data)
                print(self.__sock)
                if data == b'':
                    self.__reconnect()
                print(data)
                if data:
                    if data != "error" and data != 'pass':
                        self.__storager.real_time_data_storage(data)
            except Exception as e:
                print(f"{datetime.now()} {self.__ip}:{self.__port} {e}")
                time.sleep(5)
                self.__connected = False
                self.__reconnect()
        else:
            self.__reconnect()


    def command_polling2(self, command_list):
        if self.__connected:
            for i in range(len(command_list)):
                command_item = command_list[i]
                # print(command_item)
                if not self.__command_queue.empty():
                    write_command = self.__command_queue.get()  # д
                    try:
                        res = self.send_command(command=write_command)
                        print(res)
                    except Exception as e:
                        print("mofbus_rtu,write[ERROR]:" + str(e))
                    # if not res:
                    #     # ݣѯԤʱʣʱ䣩ط
                    #     sent_times = 1
                    #     while sent_times < resend_times:
                    #         res = self.exec_command(write_command)
                    #         sent_times += 1
                    #         if res:
                    #             break
                else:
                    try:
                        time.sleep(0.2)
                        size = command_item["size"]
                        if isinstance(command_item, dict) and "close_query_status" in command_item.keys():
                            # 鿴ˮ̨Ƶ״̬
                            start_query_status = command_item["start_query_status"]      # ѯಿֹƵʼƿ״ָ̬
                            close_query_status = command_item["close_query_status"]      # رղѯָ

                            result = self.send_command({"size": size, "command": start_query_status})

                            status = 1
                            while status:
                                try:
                                    self.__sock.recv(size)
                                    self.send_command(close_query_status)
                                except:
                                    status = 0
                        elif "command" in command_item.keys():
                            result = self.send_command({"size": size, "command": command_item["command"]})
                            # result = b':\xb3\x1a?\x00\x00\x00B?\x80\xcd\x00<\x01\x01\x01\xf1\xf6'
                            # print(result)

                        if result:
                            # print("result = ", result)
                            format_data = self.__converter.convert(self.__data_point_config, result)
                            if format_data:
                                if format_data != "error" and format_data != 'pass':
                                    # redis洢
                                    self.__storager.real_time_data_storage(format_data)


                    except Exception as e:
                        print(datetime.now(), self.__ip, "[tcp_connector]command_polling2 error: ", e)
                        time.sleep(5)
                        # self.__reconnect()x
        else:
            self.__reconnect()

