#!/usr/bin/env python
# -*- coding:utf-8 -*-

from converter import Converter
from log import OutPutLog

class S7Converter(Converter):

    def __init__(self):
        self._log = OutPutLog()

    def convert(self, config, data, master):
        # print(data)
        if data:
            format_data_dict = {}  # 列表数据转换字典数字
            try:
                for index in config:
                    address = index['address']
                    if address.startswith("DB"):
                        # data =  {'DB13': bytearray(b"A\xc8\x00\x00A\xd2A\xeeB\x08\x00\x00=tq\xc7B\x00\x00\x00B\x06\x86\xe3?\x04\x12\xb9?\x0c\xcc\xcd=L\xcc\xcd>\xcc\xcc\xcd?\x00\x00\x00\x00\x00\x01\r>\xa8j1?\x0c\xcc\xcd=L\xcc\xcd=\xcc\xcc\xcd?\x00\x00\x00\x00\x00\x01\x1a\x00\x00\x00\x01?\r\x88p?\x80\x00\x00>L\xcc\xcd?333?5\xc2\x8fA\x07\xb8\x90A\xa0\x00\x00\x00\x00\x00\x00@KC\x96\x00\x00\x00\x00A\xce\xcc\xcdB\x0c\x00\x00Ap\x00\x00B?\xa3\xd7B\xc6\x00\x00\x00\x00\x00\x00A\xc8\x00\x00A \x00\x00A\xbc\x1a\xe0A\xd0\x00\x00A\x80\x00\x00A\x10v\xfb?\x88X\x82Ap\x00\x00A\x08\x00\x00@\xa0\x00\x00A\xa6\x14{A\x18\x00\x00@\xe2p\x92A\x10\x00\x00\x00\x00\x00\x00Az\xdc\x80Dy\xc0\x00\xc4y\xc0\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00A\xcaJp\x00\x00\x00\x00\x00\x00\x00\x00A\x06\x9aGAp\x00\x00@\xa0\x00\x00@\xc9\xd8\xe4A`\x00\x00\x00\x00\x00\x00\xc1\xb2d\x80Dy\xc0\x00\xc4y\xc0\x00A\xc8\x00\x00A\xc8\x00\x00A\xc8\x00\x00A\xf4\x00\x00A\xc8\x00\x00\x01\x00Ap\x00\x00\x01\x00\x19\x0f\x01\x00Cf\xb33ChfgCi\xb33C\xc8\x8c\xcdC\xc9&gC\xc9\xd9\x9aA)G\xaf@\xe4\xcc\xce?\xd9\x99\x9bElF{\x00\x00\x00\x00\x00Z@\xdc\xcc\xcd?fffC\x05\x00\x00D\x18@\x00\x00\x01\x00\x00\x00\x00\x00Z\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\'GJ\x00D\x80\xc3\x04\x00\n\x90\x82\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00@\x88\xa3\xd7\x00\x00?^\xa1T?\x80\x00\x00>L\xcc\xcd\x04\x00A\xca\x0f\x00")}
                        addr = self.address_transform(address)     # 以 DB13.DBX395.1 为例：DB13.395.1
                        addr_division = "".join(addr).split(".")            # ['DB13', '395', '1']
                        db_number = addr_division[0]                        # DB13
                        data_addr = int(addr_division[1])                   # 395

                    elif address.startswith("M") or address.startswith("I"):
                        # data =  {'M0': bytearray(b'\xc0\x04')}
                        addr_division = address.split(".")  # 以"M20.6"为例，["M20", "6"]
                        db_number = addr_division[0]
                        data_addr = 0

                    elif address.startswith("VD") or address.startswith("VW"):
                        db_number = address
                        data_addr = 0

                    if db_number in data:
                        if data[db_number] != None : #防止设备断线返回为空导致的异常
                            if data_addr < len(data[db_number]):
                                type = index['data_type']
                                if type == "FLOAT32":
                                    return_data = master.byteTransform.TransSingle(data[db_number], data_addr)
                                elif type == "BOOL":
                                    return_data = int(master.byteTransform.TransBoolArray(data[db_number], data_addr, 1)[int(addr_division[-1])])
                                    # return_data = int(master.byteTransform.TransBoolArray(data[db_number], data_addr, 1)[int(addr_division[2])])
                                elif type == "UINT8":
                                    return_data = master.byteTransform.TransByte(data[db_number], data_addr)
                                elif type == "USINT":
                                    return_data = master.byteTransform.TransByte(data[db_number], data_addr)
                                elif type == "INT32":
                                    # INT32：DInt类型
                                    return_data = master.byteTransform.TransInt32(data[db_number], data_addr)
                                elif type == "INT16":
                                    # INT16：Int类型
                                    return_data = master.byteTransform.TransInt16(data[db_number], data_addr)

                                if isinstance(return_data, float):
                                    return_data = round(return_data, 2)
                                name = 'c' + str(index['serial_number'])
                                format_data_dict[name] = return_data

                # print("format_data_dict", format_data_dict)
            except Exception as e:
                print(data.keys(), "S7Converter[ERROR]:"+str(e))
                return "error"
            return format_data_dict

    def address_transform(self, addr):
        real_addr_buffer = "".join(addr).split(".")  # 合并list为字符串 用“.”分割字符串获得数据
        if real_addr_buffer[1][2] == 'X':
            data_addr = real_addr_buffer[0] + "." + real_addr_buffer[1][3:] + "." + real_addr_buffer[2]
        elif real_addr_buffer[1][2] == 'D' or real_addr_buffer[1][2] == 'B' or real_addr_buffer[1][2] == 'W':
            data_addr = real_addr_buffer[0] + "." + real_addr_buffer[1][3:]
        return data_addr
