"""
@Date  :2021/6/16/14:47
@Author:zz
@Desc  :投喂数据统计
"""

import time
import json
import threading
import traceback

from event_storage import EventStorage
from utility import Utility
from log import OutPutLog
from apis.get_feeding_advise import GetFeedingAdvise


class FeedingStatistics:
    def __init__(self):
        self._storage = EventStorage()
        self._log = OutPutLog()
        self._run_time = None
        self._feed_id = None

    def str_to_flaot(self, str):
        if str != None:
            return float(str)
        else:
            return 'null'

    # 历史存储主函数
    def run(self):
        self._log.info('[AutoFeeding] - FeedingStatistics module is running!')
        # 风向：c1，风速：c2,流速：c7，流向：c8
        # 罗茨风机1号运行频率：c441
        # 罗茨风机2号运行频率：c442
        # 罗茨风机3号运行频率：c443
        # 罗茨风机4号运行频率：c444
        # 旋转输料器1号运行频率：c445
        # 旋转输料器2号运行频率：c446
        # 旋转输料器3号运行频率：c447
        # 旋转输料器4号运行频率：c448
        # 线路4-1运行：c572
        # 线路4-2运行：c573
        # 线路4-3运行：c574
        # 线路4-4运行：c575
        # 线路4-5运行：c576
        list = ["c1", "c2", "c7", "c8", "c441", "c442", "c443", "c444", "c445", "c446", "c447", "c448", "c572", "c573", "c574", "c575", "c576"]

        # 获取配置信息
        while 1:
            time.sleep(2)
            real_data_dict = self._storage.get_real_data(list)

            for key in real_data_dict:
                if real_data_dict[key] == None:
                    real_data_dict[key] = 0

            # 记录投喂过程
            try:
                feeder_frequency = 0
                # 判断供料器和投饵口
                if float(real_data_dict['c445']) > 1:
                    feeder_frequency = float(real_data_dict['c445'])
                    tank_number = 1
                    feeding_port = 101
                elif float(real_data_dict['c446']) > 1:
                    feeder_frequency = float(real_data_dict['c446'])
                    tank_number = 2
                    feeding_port = 201
                elif float(real_data_dict['c447']) > 1:
                    feeder_frequency = float(real_data_dict['c447'])
                    tank_number = 3
                    feeding_port = 301
                elif float(real_data_dict['c448']) > 1:
                    feeder_frequency = float(real_data_dict['c448'])
                    tank_number = 4
                    if int(real_data_dict['c572']) == 1:
                        feeding_port = 401
                    elif int(real_data_dict['c573']) == 1:
                        feeding_port = 402
                    elif int(real_data_dict['c574']) == 1:
                        feeding_port = 403
                    elif int(real_data_dict['c575']) == 1:
                        feeding_port = 404
                    elif int(real_data_dict['c576']) == 1:
                        feeding_port = 405
                else:
                    self._feed_id = None

                # 判断风机
                if float(real_data_dict['c441']) > 0:
                    fen_frequency = float(real_data_dict['c441'])
                elif float(real_data_dict['c442']) > 0:
                    fen_frequency = float(real_data_dict['c442'])
                elif float(real_data_dict['c443']) > 0:
                    fen_frequency = float(real_data_dict['c443'])
                elif float(real_data_dict['c444']) > 0:
                    fen_frequency = float(real_data_dict['c444'])

                if feeder_frequency > 1:
                    now_time = time.time()
                    if self._feed_id == None:
                        self._feed_id = 1
                        feed_id = int(now_time)
                    if self._run_time == None:
                        self._run_time = int(now_time)
                    save_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(now_time))
                    wind_speed = self.str_to_flaot(real_data_dict['c2'])
                    wind_direction = self.str_to_flaot(real_data_dict['c1'])
                    flow_speed = self.str_to_flaot(real_data_dict['c7'])
                    flow_direction = self.str_to_flaot(real_data_dict['c8'])

                    sql_get_task_info = "SELECT * FROM touer_set;"
                    task_info = self._storage.execute_sql(sql_get_task_info)
                    task_info = task_info[0]
                    if task_info['feed_mode'] == 1:
                        feeding_mode = "自动"
                    elif task_info['feed_mode'] == 0:
                        feeding_mode = "手动"
                    else:
                        feeding_mode = None
                    breed = task_info['feed_kind']
                    tank_number_info = eval(task_info["tank1"])
                    bait_name = tank_number_info['name']
                    feed_weight = float(tank_number_info['weight'])
                    feed_diameter = int(tank_number_info['diameter'])
                    shelf_life = tank_number_info['shelf_life']
                    feeding_speed = 1440 / 67 * feeder_frequency / 50 * (8 - 0.5 * feed_diameter) * 1 / 60
                    now_time = int(time.time())
                    feeding_amount = round(feeding_speed * (now_time - self._run_time), 2)
                    self._run_time = now_time

                    sql = "INSERT INTO fishery_data(feed_id, times, wind_speed, wind_direction,\
                           flow_speed, flow_direction, feeding_port, tank_number, feeder_frequency,\
                           fan_frequency, feeding_amount, feeding_mode, breed, bait_name, feed_weight, feed_diameter, shelf_life) \
                           VALUES (%s,'%s',%s,%s,%s,%s,%s,%s,%s,%s,%s,'%s','%s','%s',%s,%s,'%s')" % \
                          (feed_id, save_time, wind_speed, wind_direction, flow_speed, flow_direction, feeding_port, \
                           tank_number, feeder_frequency, fen_frequency, feeding_amount, feeding_mode, breed, bait_name,
                           feed_weight, feed_diameter, shelf_life)

                    #print(sql)
                    self._storage.execute_sql(sql)
                else:
                    self._run_time = None
                    self._feed_id = None
            except Exception as e:
                print('行号', e.__traceback__.tb_lineno)
                print("get fishery_data [ERROR]:" + str(e))
                self._log.error("get fishery_data [ERROR]:" + str(e))
